package q1k2.pxlx.train.plugin.form;

import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.LocaleString;
import kd.bos.entity.datamodel.events.PackageDataEvent;
import kd.bos.filter.CommonFilterColumn;
import kd.bos.form.events.FilterContainerInitArgs;
import kd.bos.form.field.ComboItem;
import kd.bos.form.operatecol.OperationColItem;
import kd.bos.list.column.ListOperationColumnDesc;
import kd.bos.list.plugin.AbstractListPlugin;
import kd.sdk.plugin.Plugin;
import q1k2.pxlx.train.common.Q1k2MonthPlanLwConstant;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;

/**
 * @Description 月度计划列表插件
 * @Created 刘维
 */
public class MonthPlanListPlugin extends AbstractListPlugin implements Plugin {
    /**
     * 过滤控件初始化
     *
     * @param args
     */
    @Override
    public void filterContainerInit(FilterContainerInitArgs args) {
        super.filterContainerInit(args);
        /****** 常用过滤视图下的筛选条件处理 ******/
        CommonFilterColumn yearFilter = new CommonFilterColumn(Q1k2MonthPlanLwConstant.Q1K2_YEAR);
        //多选
        yearFilter.setMulti(true);
        // 设置标题
        yearFilter.setCaption(new LocaleString("年度"));
        Calendar calendar = Calendar.getInstance();
        int year = calendar.get(Calendar.YEAR);//当前年度
        // 设置下拉值
        List<ComboItem> data = new ArrayList<>();
        for (int i = year - 3; i <= year + 3; i++) {
            data.add(new ComboItem(new LocaleString(String.valueOf(i)), String.valueOf(i)));
        }
        yearFilter.setComboItems(data);
        args.addFilterColumn(yearFilter);
    }

    /**
     * 数据包装事件,用于修改发送给前端之前的数据，用于单元格填值
     *
     * @param evt 数据包装事件参数 PackageDataEvent
     */
    @Override
    public void packageData(PackageDataEvent evt) {
        super.packageData(evt);
        this.setOperationColItemStyle(evt);
    }

    /**
     * 操作列控制(某个操作项上显示名称)
     *
     * @param evt
     */
    private void setOperationColItemStyle(PackageDataEvent evt) {
        Object obj = evt.getSource();
        // 获取当前行数据
        DynamicObject rowData = evt.getRowData();
        // 判断是否是操作列
        if (obj instanceof ListOperationColumnDesc) {
            String billstatus = rowData.getString("billstatus");// 状态
            List<OperationColItem> operationColItems = (List<OperationColItem>) evt.getFormatValue();
            for (OperationColItem item : operationColItems) {
                //操作键
                String operationKey = item.getOperationKey();
                if (("modify".equalsIgnoreCase(operationKey)
                        || "submit".equalsIgnoreCase(operationKey))
                        && "A".equals(billstatus)) {
                    //暂存状态 显示 修改和提交
                    item.setVisible(true);
                    //item.setLocked(true);
                } else if (("unsubmit".equalsIgnoreCase(operationKey)
                        || "changeplan".equalsIgnoreCase(operationKey)
                        || "execstatus".equalsIgnoreCase(operationKey))
                        && "B".equals(billstatus)) {
                    //已提交状态 显示 撤销、变更计划、实施情况
                    item.setVisible(true);
                }
            }
        }
    }
}