package q1k2.pxlx.train.plugin.form;

import com.kingdee.util.StringUtils;
import kd.bos.bill.OperationStatus;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.LocaleString;
import kd.bos.entity.datamodel.ListSelectedRowCollection;
import kd.bos.entity.datamodel.events.PackageDataEvent;
import kd.bos.filter.CommonFilterColumn;
import kd.bos.form.FormShowParameter;
import kd.bos.form.OpenStyle;
import kd.bos.form.ShowType;
import kd.bos.form.events.AfterDoOperationEventArgs;
import kd.bos.form.events.FilterContainerInitArgs;
import kd.bos.form.field.ComboItem;
import kd.bos.form.operatecol.OperationColItem;
import kd.bos.list.IListView;
import kd.bos.list.column.ListOperationColumnDesc;
import kd.bos.list.plugin.AbstractListPlugin;
import kd.sdk.plugin.Plugin;
import q1k2.pxlx.train.common.Q1k2YearPlanConstant;

import java.util.*;

/**
 * @Description 年度计划列表插件
 * @Created 刘维
 */
public class YearPlanListPlugin extends AbstractListPlugin implements Plugin {
    //查看明细
    private String VIEWDETAIL = "viewdetail";

    /**
     * 过滤控件初始化
     *
     * @param args
     */
    @Override
    public void filterContainerInit(FilterContainerInitArgs args) {
        super.filterContainerInit(args);
        /****** 常用过滤视图下的筛选条件处理 ******/
        CommonFilterColumn yearFilter = new CommonFilterColumn(Q1k2YearPlanConstant.Q1K2_YEAR);
        //多选
        yearFilter.setMulti(true);
        // 设置标题
        yearFilter.setCaption(new LocaleString("年度"));
        Calendar calendar = Calendar.getInstance();
        int year = calendar.get(Calendar.YEAR);//当前年度
        // 设置下拉值
        List<ComboItem> data = new ArrayList<>();
        for (int i = year - 3; i <= year + 3; i++) {
            data.add(new ComboItem(new LocaleString(String.valueOf(i)), String.valueOf(i)));
        }
        yearFilter.setComboItems(data);
        args.addFilterColumn(yearFilter);
    }

    /**
     * 数据包装事件,用于修改发送给前端之前的数据，用于单元格填值
     *
     * @param evt 数据包装事件参数 PackageDataEvent
     */
    @Override
    public void packageData(PackageDataEvent evt) {
        super.packageData(evt);
        this.setOperationColItemStyle(evt);
    }

    /**
     * 操作列控制(某个操作项上显示名称)
     *
     * @param evt
     */
    private void setOperationColItemStyle(PackageDataEvent evt) {
        Object obj = evt.getSource();
        // 获取当前行数据
        DynamicObject rowData = evt.getRowData();
        // 判断是否是操作列
        if (obj instanceof ListOperationColumnDesc) {
            String billstatus = rowData.getString("billstatus");// 状态
            List<OperationColItem> operationColItems = (List<OperationColItem>) evt.getFormatValue();
            for (OperationColItem item : operationColItems) {
                //操作键
                String operationKey = item.getOperationKey();
                if (("modify".equalsIgnoreCase(operationKey)
                        || "submit".equalsIgnoreCase(operationKey))
                        && "A".equals(billstatus)) {
                    //暂存状态 显示 修改和提交
                    item.setVisible(true);
                    //item.setLocked(true);
                } else if (("unsubmit".equalsIgnoreCase(operationKey)
                        || "viewdetail".equalsIgnoreCase(operationKey)
                        || "process".equalsIgnoreCase(operationKey)
                        || "split_month_plan".equalsIgnoreCase(operationKey))
                        && "B".equals(billstatus)) {
                    //已提交状态 显示 撤销、查看明细、执行情况、拆分月度计划
                    item.setVisible(true);
                }
            }
        }
    }

    @Override
    public void afterDoOperation(AfterDoOperationEventArgs args) {
        super.afterDoOperation(args);
        // 查看明细
        if (StringUtils.equals(VIEWDETAIL, args.getOperateKey())) {
            // 获取列表
            IListView listView = (IListView) this.getView();
            //获取当前选中所有行
            ListSelectedRowCollection selectedRows = listView.getSelectedRows();
            //单据主键ids
            Object[] primaryKeyValues = selectedRows.getPrimaryKeyValues();
            //构造ListShowParameter，打开子页面弹窗
            //父页面传递参数
            FormShowParameter showParameter = new FormShowParameter();
            showParameter.setFormId("q1k2_year_plan_detail_lw");
            //传递单个参数
            showParameter.setCustomParam("ids", primaryKeyValues);
            //设置单据主键
            //showParameter.setPkId(selectedRow.getPrimaryKeyValue());
            OpenStyle openStyle = showParameter.getOpenStyle();
            openStyle.setShowType(ShowType.Modal);//设置打开方式
            //打开状态:新增
            showParameter.setStatus(OperationStatus.VIEW);
            this.getView().showForm(showParameter);//通过showForm打开新页面
        }
    }
}